--------------------------------------------------------------------------------
-- Copyright (C) 2004 Trenz Electronic GmbH
-- www.trenz-electronic.de
-- Author:      Tarek Schlabeck
--------------------------------------------------------------------------------
-- Project:     TE-XC3S Application Note: Pong Demo
-- File:        lcd.vhd
-- Description: LCD control for the Pong Demo
-- History:     V1.0 2004-10-13 TS created
--------------------------------------------------------------------------------
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

	-- this component initializes the display and executes simple write commands
	-- for display output
   
entity lcd is
    Port ( clk,res : in std_logic;
    		clk_25us,clk_50us: in std_logic;
	 		  write : in std_logic;
			  data : in std_logic_vector(7 downto 0);
			  ready : out std_logic;

			  -- lcd signals   ------------
			  lcd_data: inout std_logic_vector(3 downto 0);
			  lcd_rw: out std_logic;
			  lcd_rs: out std_logic;
			  lcd_en: out std_logic);
end lcd;


architecture behavioral of lcd is

 type lcd_state_type is (wait_1,wait_2,wait_3,wait_4,wait_5,wait_6,setup_1,setup_2,setup_3
 					,setup_4,setup_5,setup_6,setup_7,setup_8,setup_9
					,setup_10,setup_11,setup_12,write_1,write_2);
 signal lcd_state: lcd_state_type;

 signal write_to_lcd: std_logic;

begin

lcd_control: process (clk)

variable count: integer range 0 to 1000;

begin
	if rising_edge(clk) then
	  if (res='0') then lcd_state<=wait_1;	    -- reset
	  				lcd_en<='0';
	  else
		if (clk_25us='1') then			    -- half clock for setting the lcd_en signal
		    if (clk_50us='1') then
		        lcd_en<='0';
		    elsif (write_to_lcd='1') then lcd_en<='1';
		    else lcd_en<='0';
		    end if;
		end if;

		if (clk_50us='1') then			    -- clocktime min. 40us because of working time of the lcd

			if (count=1000) then count:=0;
			else count:=count+1;		   -- counter for waiting times
			end if;

		 	if (lcd_state=wait_5) then ready<='1';   -- set ready flag only in one state
			else ready<='0';
			end if;

	    -- state-machine 
		 	case lcd_state is

				when wait_1 =>  lcd_data<="0000";		-- wait for min. 40ms after power on
							 lcd_rs<='0';			-- register mode
							 lcd_rw<='0';			-- write mode
							 write_to_lcd<='0';
							 
							if (count=1000) then lcd_state<=setup_1;
							else lcd_state<=wait_1;
							end if;

				when setup_1 => lcd_state<=wait_2;	   -- set 8-Bit mode
							 lcd_data<="0011";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when wait_2 => write_to_lcd<='0';	  -- wait for min. 4.1ms
							if (count=200) then lcd_state<=setup_2;
							else lcd_state<=wait_2;
							end if;

				when setup_2 => lcd_state<=wait_3;		-- set 8-Bit mode
							 lcd_data<="0011";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when wait_3 => write_to_lcd<='0';		-- wait for min. 100us
							if (count=300) then lcd_state<=setup_3;
							else lcd_state<=wait_3;
							end if;

				when setup_3 => lcd_state<=setup_4;    -- set 8-Bit mode
							 lcd_data<="0011";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when setup_4 => lcd_state<=setup_5;	-- set 4-Bit mode
							 lcd_data<="0010";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when setup_5 => lcd_state<=setup_6;	-- now 4-Bit mode is set
							 lcd_data<="0010";		-- now sending higher nibble first and
							 lcd_rs<='0';			-- lower nibble as second
							 lcd_rw<='0';
							 write_to_lcd<='1';		-- set 4-Bit mode, display lines and dots

				when setup_6 => lcd_state<=setup_7;
							 lcd_data<="1000";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when setup_7 => lcd_state<=setup_8;	-- display, cursor, blinking character on
							 lcd_data<="0000";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when setup_8 => lcd_state<=setup_9;
							 lcd_data<="1111";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when setup_9 => lcd_state<=setup_10;	-- clear display
							 lcd_data<="0000";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when setup_10 =>lcd_state<=wait_4;
							 lcd_data<="0001";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when wait_4 =>	write_to_lcd<='0';	    -- wait for clear time (min. 1.64ms)
							if (count=400) then lcd_state<=setup_11;
							else lcd_state<=wait_4;
							end if;

				when setup_11 =>lcd_state<=setup_12;  -- set direction of cursor movement
							 lcd_data<="0000";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when setup_12 =>lcd_state<=wait_5;
							 lcd_data<="0110";
							 lcd_rs<='0';
							 lcd_rw<='0';
							 write_to_lcd<='1';

				when wait_5  =>write_to_lcd<='0';	  -- wait for write command from outside
							
							if (write='1') then lcd_state<=write_1;
							else lcd_state<=wait_5;
							end if;

				when write_1 =>lcd_state<=write_2;	  -- write higher nibble
							lcd_data<=data(7 downto 4);
							lcd_rs<='1';	-- switch to text mode
							lcd_rw<='0';
							write_to_lcd<='1';
							

				when write_2 =>lcd_state<=wait_6;	  -- write lower nibble
							lcd_data<=data(3 downto 0);
							lcd_rs<='1';
							lcd_rw<='0';
							write_to_lcd<='1';

				when wait_6 => write_to_lcd<='0';	 -- wait for not write command
							if (write='0') then lcd_state<=wait_5;	  
							else lcd_state<=wait_6;
							end if;
							
							
				end case;
		   end if;
		end if; 
	   end if;
  end process;
  



end behavioral;