--------------------------------------------------------------------------------
-- Copyright (C) 2004 Trenz Electronic GmbH
-- www.trenz-electronic.de
-- Author:      Kolja Sulimma
--------------------------------------------------------------------------------
-- Project:     TE-XC3S Application Note: Pong Demo
-- File:        ps2.vhd
-- Description: PS/2 input for the Pong Demo
-- History:     V1.0 2004-10-11 KS created
--------------------------------------------------------------------------------
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- this component processes the PS2 keyboard protocol and keep track whether certain keys are pressed.
-- currently the keys are cursor righ, left, up and down, keypad 0 and space but this can easily be changed.
-- this component can handle any number of simultaneausly pressed keys, but the keyboards usually only support two.
   
entity ps2 is
    Port ( clk : in std_logic;
	 		  ps2_clk : in std_logic;
			  ps2_data : in std_logic;
			  right, left, up, down, kp0, space : out std_logic
			  
			 );
end ps2;


architecture behavioral of ps2 is
signal ps2_clk_1, ps2_clk_2, break : std_logic;
signal  shiftreg, notshiftreg : std_logic_vector(9 downto 0);

begin


-- the shift registers stores the inverted PS2 data. This way it can be initialized to 0 which is the
-- default for xilinx FPGAs.
notshiftreg <= not shiftreg;

process (clk)
begin
	if rising_edge(clk) then
		-- synchronize PS2 clock to internal clock
		ps2_clk_1 <= ps2_clk;
		ps2_clk_2 <= ps2_clk_1;

		if (ps2_clk_2 = '1' and ps2_clk_1 = '0') then  --ps2_clk falling edge
		   -- when the start bit reached the lowest bit of the register the scan code
			-- can be evaluated and the register can be cleared again.
			if (notshiftreg(0) = '0') then
				shiftreg <= "0000000000";
				if notshiftreg(8 downto 1) = "11110000" then
 			   	break <= '1';
				else 
					break <= '0';
				end if;

				case notshiftreg(8 downto 1) is
					when "01101011" => --6B, kp 4
 				   	left <= not break;
					when "01110100" => --74, kp 6
 				   	right <= not break;
					when "01110101" => --75, kp 8
 				   	up <= not break;
					when "01110010" => --72, kp 2
						down <= not break;
					when "01110000" => --70, kp0
						kp0 <= not break;
					when "00101001" => --29, space
						space <= not break;
					
					when others =>
						null;
				 end case;
			else 
				shiftreg <= (not ps2_data) & shiftreg(9 downto 1);
			end if;

		end if;
	end if;
 end process;
end behavioral;