--------------------------------------------------------------------------------
-- Copyright (C) 2004 Trenz Electronic GmbH
-- www.trenz-electronic.de
-- Author:      Kolja Sulimma
--------------------------------------------------------------------------------
-- Project:     TE-XC3S Application Note: Pong Demo
-- File:        sprites.vhd
-- Description: Sprites for the Pong Demo
-- History:     V1.0 2004-10-11 KS created
--------------------------------------------------------------------------------
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- sprite components
-- you can display colored rectangles
-- using the hit signal, but the address signal can also
-- be used to display a sprite image from a rom	

entity sprite is
	Generic (
				shift_x : integer := 0;	-- scale in x coordinate by 2**shift_x
				shift_y : integer := 0;	-- scale in y coordinate by 2**shift_y
				bits_x  : integer;  		-- screen size in x coordinate is 2**(shift_x + bits_x)
				size_y  : integer		   -- screen size in y coordinate
	);
    Port ( screen_x : in std_logic_vector(8 downto 0);   -- position cathode ray on screen
           screen_y : in std_logic_vector(8 downto 0);
           pos_x : in std_logic_vector(8 downto 0);	   -- lower right corner of sprite on screen
           pos_y : in std_logic_vector(8 downto 0);
           hit : out std_logic;	 								-- '1' if cathode ray is inside sprite rectangle 
           address : out std_logic_vector(14 downto 0));
			  --address is the relative address in memory
			  --to the beginning of the sprite data

end sprite;

architecture Behavioral of sprite is


begin
hit <= '1' when (screen_x >= pos_x) and 
					 (screen_y >= pos_y) and
					 (screen_y <  (pos_y+size_y)) and
					 ("0"&screen_x <  ("0"&pos_x+ to_stdlogicvector("0000000001" sll bits_x)))
 					 --(screen_y <  pos_y+size_y)
			  else '0';

--address is only valid when hit is set
address <= to_stdlogicvector("000000"&to_bitvector(screen_y - pos_y) sll (bits_x - shift_y)) or
           to_stdlogicvector("000000"&to_bitvector(screen_x - pos_x) srl shift_x);
          
end Behavioral;
