--------------------------------------------------------------------------------
-- Copyright (C) 2004 Trenz Electronic GmbH
-- www.trenz-electronic.de
-- Author:      Kolja Sulimma
--------------------------------------------------------------------------------
-- Project:     TE-XC3S Application Note: Pong Demo
-- File:        sram.vhd
-- Description: SRAM memory test for the Pong Demo
-- History:     V1.0 2004-10-13 KS created
--------------------------------------------------------------------------------
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity sram is port(
		clk : in std_logic;
		reset : in std_logic;
		led : out std_logic_vector( 1 to 4);
		ram_a : out std_logic_vector(18 downto 1);
		ram_io : inout std_logic_vector(15 downto 0);
		ram_ce : out std_logic;
		ram_oe : out std_logic;
		ram_we : out std_logic;
		ram_bhe : out std_logic;
		ram_ble : out std_logic );
end sram;

architecture behavioral of sram is begin

-- sram
-- The sram is not used by this demo. Instead a LFSR test is implemented.
-- Pseudo random numbers are written to the ram and read back. If a mismatch
-- is detected one of the LEDs is turned off.
process (clk,reset)

type sram_state_type is (write_init, write1, write2, read_init, read1);
variable sram_state, next_sram_state : sram_state_type;
variable lfsr, last_lfsr : std_logic_vector(18 downto 0);
variable last_io : std_logic_vector(15 downto 0);
variable address : std_logic_vector(17 downto 0);

begin
ram_ce <= '0';
ram_bhe <= '0';
ram_ble <= '0';
ram_a <= address;
if reset = '0' then
  address := "000000000000000000";
  lfsr := "0000000000000000001";
  sram_state := write_init;
  ram_we <= '1';
  ram_io <= "ZZZZZZZZZZZZZZZZ";
  led <= "1100";
elsif	rising_edge(clk) then
	case sram_state is 
		when write_init =>
			sram_state := write1;
			lfsr := "0000000000000000001";
			ram_io <= "0000000000000001";
			ram_we <= '1';
			ram_oe <= '1';
			address := "000000000000000000";
			led(1) <= '0';
			
			led(4) <= '0';

		when write1 =>
			sram_state := write2;
			ram_we <= '0';
			ram_io <= lfsr(15 downto 0);
		when write2 =>
			
			if address = "11"&X"FFFF" then
				sram_state := read_init;
				address := "000000000000000000";
			else
				sram_state := write1;
				address := address + 1;
			end if;
			
			lfsr := lfsr(17 downto 0) & (lfsr(18) xnor lfsr(17));
			ram_we <= '1';
		when read_init =>		--address is allready 0
			ram_oe <= '0';
 			sram_state := read1;
			lfsr := "0000000000000000001";
			ram_io <= "ZZZZZZZZZZZZZZZZ";
			led(4) <= '1';
			led(2) <= '0';
	
		when read1 =>
			if address = "11"&X"FFFF" then
				sram_state := write_init;
			end if;
	
			if ram_io(15 downto 0) /= lfsr(15 downto 0) and address /= "00"&X"0000" then
				led(1) <= '1';
				led(2) <= '1';
				led(3) <= '1';
			else
				--led(1) <= '0';
			end if;

			last_lfsr := lfsr;
			last_io := ram_io;
			address := address + 1;
			lfsr := lfsr(17 downto 0) & (lfsr(18) xnor lfsr(17));
	end case;

end if;
end process;

end behavioral;
