--------------------------------------------------------------------------------
-- Copyright (C) 2004 Trenz Electronic GmbH
-- www.trenz-electronic.de
-- Author:      Kolja Sulimma
--------------------------------------------------------------------------------
-- Project:     TE-XC3S Application Note: Pong Demo
-- File:        vga.vhd
-- Description: VGA timing for the Pong Demo
-- History:     V1.0 2004-10-11 KS created
--------------------------------------------------------------------------------
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--vga background information:
--http://www.hut.fi/Misc/Electronics/faq/vga2rgb/timings.html
--calculator
--http://www.hut.fi/Misc/Electronics/faq/vga2rgb/calc.html

--the strange aspect ratio has a simple reason:
--the DCMs do not allow for clock slower than 24MHz.

entity timing1024x384 is
    Port ( clk : in std_logic;  -- 40MHz Clock
           x : out std_logic_vector(9 downto 0);
           y : out std_logic_vector(8 downto 0);
			  vsync_b, hsync_b: inout std_logic;
			  framestrobe : out std_logic;
           blank : out std_logic);
end timing1024x384;

architecture Behavioral of timing1024x384 is
signal xcount : std_logic_vector(10 downto 0);
signal ycount : std_logic_vector(8 downto 0);
begin

x <= xcount(9 downto 0);
y <= ycount(8 downto 0);

blank <= '1' when ((xcount(10) = '1') or (ycount(8 downto 7) = "11")) else '0';
hsync_b <= '0' when ((xcount >= 1024+82) and (xcount < 1024+82+156)) else '1';
vsync_b <= '0' when ((ycount = 415) or (ycount = 416)) else '1';

process (clk)
begin
if rising_edge(clk) then
	if (xcount = 0) then 
		xcount <= conv_std_logic_vector(1024 + 32 + 156 + 82, 11);
		if (ycount = 0) then
			framestrobe <= '1';
			ycount <= conv_std_logic_vector(384 + 10 + 2 + 31, 9);
		else
			framestrobe <= '0';
			ycount <= ycount - "000000001";
		end if;
	else
		framestrobe <= '0';
		xcount <= xcount - "0000000001";
	end if;
end if;
end process;

end Behavioral;
